//AIServer.java

/***
 Questa classe implementa il metodo main() dell'AI server del gioco massacritica.
 
 Autore: Alessandro Caviola
***/

import java.io.*;

class AIServer 
{  
  static Server server = new Server();
  
  public static void main(String[] argv)
  {   
    for(int i = 0; i < argv.length; i++) //Interpretiamo i parametri dalla linea di comando
    {
      if(argv[i].compareTo("-porti") == 0)
      {
        i++;
        server.port2 = Integer.parseInt(argv[i]);
      }
      else if(argv[i].compareTo("-porto") == 0)
      {
        i++;
        server.port = Integer.parseInt(argv[i]);
      }
      else if(argv[i].compareTo("-addr") == 0)
      {
        i++;
        server.addr = new String(argv[i]);
      }
      else if(argv[i].compareTo("-depth") == 0)
      {
        i++;
        server.tree_d = Integer.parseInt(argv[i]);
      }
      else if(argv[i].compareTo("-test") == 0)
      {
        server.test = true;
      }
      else if(argv[i].compareTo("-diff") == 0)
      {
        i++;
        server.diff = Integer.parseInt(argv[i]);
        System.out.println("Il parametro -diff non e' disponibile in questa versione.");
      }
      else if(argv[i].compareTo("-oldp") == 0)
      {
        server.oldp = true;
        server.term = null;
      }
      else if(argv[i].compareTo("-verbose") == 0)
      {
        server.verbose = true;
      }
      else if(argv[i].compareTo("-term") == 0)
      {
        i++;
        
        if(server.oldp) 
        {
          server.term = null;
          continue;
        }
        int term = Integer.parseInt(argv[i]);
        if(term == 0) server.term = new String(":");
        else if(term == 1) server.term = new String("\\0");
        else if(term == 2) server.term = new String("\\n");
        else if(term == 3) server.term = new String("-");
      }
      else if(argv[i].compareTo("-console") == 0)
      {
        server.console = true;
      }
      else if(argv[i].compareTo("-onetime") == 0)
      {
        server.onetime = true;
      }
      else if(argv[i].compareTo("-runhelp") == 0)
      {
        System.out.println("Opzioni di console AIServer v.1.2:");
        System.out.println("q: Termina il programma");
        System.out.println("v: Inverte l'impostazione verbose (attiva se e' disattivata e viceversa)");
        System.out.println("s: Visualizza alcune informazioni riguardanti la partita");
        System.out.println("+: Incrementa la profondita' massima dell'albero");
        System.out.println("-: Decrementa la profondita' massima dell'albero");
        System.out.println("h: Visualizza questo testo");
        System.out.println("");
      }
      else if(argv[i].compareTo("-?") == 0 || argv[i].compareTo("-h") == 0 || argv[i].compareTo("-help") == 0)
      {
        System.out.println("Opzioni AIServer v.1.2:");
        System.out.println("-porti n: n e' la porta per il canale di input. Default: 1976.");
        System.out.println("-porto n: n e' la porta per il canale di output. Default: 1981.");
        System.out.println("-addr n: n e' l'indirizzo IP per il canale di output. Default: 127.0.0.1.");
        System.out.println("-depth n: n e' la profondita' massima del MinMax Tree. Default: 3.");
        System.out.println("-test: mostra il numero di metamosse calcolate ed il tempo richiesto.");
        System.out.println("-dif n: specifica il livello di difficolta', da 1 a 3. Default: 3.");
        System.out.println("-oldp: disattiva l'utilizzo delle estensioni al protocollo base.");  
        System.out.println("-verbose: visualizza un rapporto piu' completo sull'esecuzione del programma.");
        System.out.println("-term n: seleziona il terminatore: ':'=0,'\\0'=1,'\\n'=2,'-'=3. Default: 1.");
        System.out.println("-runhelp: visualizza le opzioni della console.");
        System.out.println("-console: esegue la console dopo l'avvio.");
        System.out.println("-onetime: termina il server automaticamente dopo la prima partita.");
        System.out.println("-? -h -help: visualizza questo testo.");
        System.out.println("");
        System.exit(0);
      }
      else
      {
        System.out.println("Parametro non valido: "+argv[i]);
        System.exit(0);
      }
    }
    
    server.Start();
    
    //Iniziamo l'interazione del terminale
    char ch;
    try
    {
      while(true)
      {
        if(server.console) System.out.print(">");
        
        ch = (char)System.in.read();
        System.in.skip(2); //skip end of line
  
        if(!server.console) 
        {
          if(ch == 'q') //Controlliamo lo stesso il carattere per l'uscita
          {
            if(server.net != null) 
            {
              String msg = server.net.CreateMsg(Net.MSG_END, 0, 0);
              server.net.SendMessage(msg);
              
              Server.Pause(250);
              
              server.net.CloseConnection();
            }
            System.exit(0);
          }
          continue; //Non elaboriamo l'input, se la console e' disattivata
        }
        
        //Eseguiamo il comando sul terminale
        switch(ch)
        {
          case 'q': //Terminiamo il programma
          {
            if(server.net != null) 
            {
              String msg = server.net.CreateMsg(Net.MSG_END, 0, 0);
              server.net.SendMessage(msg);
              try
              {
                Thread.sleep(500); //Lasciamo il tempo di spedire il messaggio
              }catch(InterruptedException e)
              {
              }
              server.net.CloseConnection();
            }              
            System.exit(0);
            break;
          }
          case 's': //Visualizziamo le informazioni
          {
            System.out.println("Pedine proprie: "+Integer.toString(server.board.GetDiscs(true)));
            System.out.println("Pedine avversarie: "+Integer.toString(server.board.GetDiscs(false)));
            System.out.println("Turno: "+Integer.toString(server.board.GetDiscs(true)));
            
            if(server.net == null) System.out.println("Nessuna partita in esecuzione");
            else System.out.println("Partita in esecuzione");
            break;
          }
          case 'v': //Modifichiamo l'opzione verbose
          {
            server.verbose = !server.verbose;
            break;
          }
          case '+': //Incrementiamo la profondita' dell'albero
          {
            server.generator.tree_depth++;
            break;
          }
          case '-': //Decrementiamo la profondita' dell'albero
          {
            server.generator.tree_depth--;
            if(server.generator.tree_depth < 1) server.generator.tree_depth = 1;
            break;
          }
          case 'h': //Visualizziamo un'aiuto
          {
            System.out.println("Opzioni di console AIServer v.1.2:");
            System.out.println("q: Termina il programma");
            System.out.println("v: Inverte l'impostazione verbose (attiva se e' disattivata e viceversa)");
            System.out.println("s: Visualizza alcune informazioni riguardanti la partita");
            System.out.println("+: Incrementa la profondita' massima dell'albero");
            System.out.println("-: Decrementa la profondita' massima dell'albero");
            System.out.println("h: Visualizza questo testo");
            System.out.println("");
          }
          default:
            System.out.println("opzione non riconosciuta!");
        }
      }
    }catch(IOException e)
    {
      System.out.println("Errore di console.");
    }                        
  } 
} 
