//XBoard.java

/***
 Questa classe estende Board includendo funzionalita' di visualizzazione del tabellone di gioco. 
 
 Autore: Alessandro Caviola
***/

import java.awt.*;
import java.awt.MediaTracker;
import javax.swing.*;

class XBoard extends Board
{
  protected Image xboard[] = new Image[5];
  protected Image discs[] = new Image[18]; 
  protected Image messages[] = new Image[4];
  protected Image end; 
  
  protected int player_ind;
  protected int opponent_ind;
  protected int board_ind;
  
  protected MediaTracker tracker;
  
  public static int COLOR_RED = 0; /***Colore rosso.***/
  public static int COLOR_BLU = 1; /***Colore blu.***/
  public static int COLOR_GREEN = 2; /***Colore verde.***/
  public static int COLOR_YELLOW = 3; /***Colore giallo.***/
  public static int COLOR_WHITE = 4; /***Colore bianco.***/
  public static int COLOR_BLACK = 5; /***Colore nero.***/
  public static int COLOR_NONE = -1; /***Nessuna informazione sul colore.***/
  
  /***Costruisce un oggetto di tipo XBoard.***/
  XBoard(Component cmp)
  {
    super();
    
    tracker = new MediaTracker(cmp);
    player_ind = COLOR_RED*3;
    opponent_ind = COLOR_BLU*3;
    board_ind = 0;
    
    end = null;
    
    //Carico i dati grafici
    xboard[0] = LoadImage("./res/board.jpg");
    tracker.addImage(xboard[0], 0);
  
    discs[0] = LoadImage("./res/red1.gif");
    tracker.addImage(discs[0], 1);    
    discs[1] = LoadImage("./res/red2.gif");
    tracker.addImage(discs[1], 2);     
    discs[2] = LoadImage("./res/red3.gif");
    tracker.addImage(discs[2], 3);  
    
    discs[3] = LoadImage("./res/blue1.gif");
    tracker.addImage(discs[3], 4);    
    discs[4] = LoadImage("./res/blue2.gif");
    tracker.addImage(discs[4], 5);     
    discs[5] = LoadImage("./res/blue3.gif");
    tracker.addImage(discs[5], 6); 
    
    discs[6] = LoadImage("./res/green1.gif");
    tracker.addImage(discs[6], 7);    
    discs[7] = LoadImage("./res/green2.gif");
    tracker.addImage(discs[7], 8);     
    discs[8] = LoadImage(".res//green3.gif");
    tracker.addImage(discs[8], 9);  
    
    discs[9] = LoadImage("./res/yellow1.gif");
    tracker.addImage(discs[9], 10);    
    discs[10] = LoadImage("./res/yellow2.gif");
    tracker.addImage(discs[10], 11);     
    discs[11] = LoadImage("./res/yellow3.gif");
    tracker.addImage(discs[11], 12);  
    
    discs[12] = LoadImage("./res/white1.gif");
    tracker.addImage(discs[12], 13);    
    discs[13] = LoadImage("./res/white2.gif");
    tracker.addImage(discs[13], 14);     
    discs[14] = LoadImage("../res/white3.gif");
    tracker.addImage(discs[14], 15);

    discs[15] = LoadImage("./res/black1.gif");
    tracker.addImage(discs[15], 16);    
    discs[16] = LoadImage("./res/black2.gif");
    tracker.addImage(discs[16], 17);     
    discs[17] = LoadImage("../res/black3.gif");
    tracker.addImage(discs[17], 18);
    
    messages[0] = LoadImage("./res/victory.gif");
    tracker.addImage(messages[0], 40);
    messages[1] = LoadImage("./res/loose.gif");
    tracker.addImage(messages[1], 41);  
    messages[2] = LoadImage("./res/victory1.gif");
    tracker.addImage(messages[2], 42);
    messages[3] = LoadImage("./res/victory2.gif");
    tracker.addImage(messages[3], 43);  
    
    try
    {
      tracker.waitForAll();
    }catch(InterruptedException e)
    {
      JOptionPane.showMessageDialog(cmp, "Impossibile caricare le immagini!", "Error", JOptionPane.ERROR_MESSAGE);
      System.err.println("Error: Impossibile caricare le immagini!");
    }
  }
  
  /***Imposta il colore per il giocatore e l'avversario. I parametri passati corrispondono alle costanti COLOR_*.***/
  public void SetPlayersColors(int player, int opponent)
  {
    if(player != COLOR_NONE) player_ind = player * 3;
    if(opponent != COLOR_NONE) opponent_ind = opponent * 3;
  }
  
  /***Restituisce il colore del giocatore uno, se player = true, dell'avversario se player = false.***/
  public int GetPlayerColor(boolean player)
  {
    if(player) return player_ind / 3;
    return opponent_ind / 3;
  }
  
  /***Imposta il tabellone di gioco.***/
  public void SetBoard(int index)
  {
    board_ind = index;
  }
  
  /***Restituisce l'indice del tabellone corrente.***/
  public int GetBoard()
  {
    return board_ind;
  }
  
  /***Carica l'immagine 'name' dal disco, e la ritorna come oggetto della classe Image.***/
  public Image LoadImage(String name)
  {
    Toolkit toolkit = Toolkit.getDefaultToolkit();
    return toolkit.getImage(name);
  }
  
  /***Visualizza il tabellone di gioco, le pedine e gli eventuali altri elementi grafici.***/ 
  public void Paint(Graphics g, boolean player1, boolean player2, boolean xboard, boolean msg)
  {
    if(xboard)g.drawImage(this.xboard[board_ind], 0, 0, null);
  
    int id = 0;
    for(int x = 0; x <= 5; x++)
      for(int y = 0; y <= 4; y++)
    {
      if(GetCurrent(board[x][y]) == 1) id = 0;
      else if(GetCurrent(board[x][y]) == 2) id = 1; 
      else if(GetCurrent(board[x][y]) == 3) id = 2;
      else continue;
    
      if(GetOwner(board[x][y]) && player1)
        g.drawImage(discs[player_ind+id], (x + 1) * 75 - 2, (y + 1) * 75 - 5, null);
    
      else if(player2)
        g.drawImage(discs[opponent_ind+id], (x + 1) * 75 - 2, (y + 1) * 75 - 5, null);
    } 
    
    //Stampiamo il messaggio di fine partita, se presente
    if(end != null && msg == true) g.drawImage(end, 150, 100, null);  
  }
  
  public void Paint(Graphics g)
  {
    Paint(g, true, true, true, true);
  }
  
  /***Visualizza un disco del giocatore 'player' alla posizione specificata dai parametri x e y.***/
  public void PaintDisc(Graphics g, int x, int y, boolean player)
  {
    if(player == true) g.drawImage(discs[player_ind], x-40, y-40, null); 
    else g.drawImage(discs[opponent_ind], x-40, y-40, null);  
  } 
  
  public void PaintDisc(Graphics g, int x, int y)
  {
    PaintDisc(g, x, y, next_move);
  }

  /***Restituisce la casella posta sotto le coordinate (x,y) dell'immagine.***/
  public Point GetHitCoords(int x, int y)
  {
    if(x < 75 || x > 525 || y < 75 || y > 450) return null;
    return new Point((x/75)-1, (y/75)-1);
  }
  
  /***Ordina di visualizzare non appena possibile le scritte corrispondenti alla vittoria o alla perdita.***/
  public void PrintVictory(boolean player, boolean mode)
  {
    if(mode == true)
    {
      if(player == true) end = messages[0];
      else end = messages[1];
    }  
    else
    {
      if(player == true) end = messages[2];
      else end = messages[3];
    }
    super.PrintVictory(player, mode);
  }
  
  /***Reinizializza il tabellone di gioco.***/
  public void Reinitialize()
  {
    super.Reinitialize();

    player_ind = COLOR_RED*3;
    opponent_ind = COLOR_BLU*3;
    board_ind = 0;
    end = null;
  }  
}